@@ -107,7 +107,7 @@ use XML::Stream 1.06;
 use IO::Select;
 use vars qw($VERSION $AUTOLOAD);
 
-$VERSION = "1.0019";
+$VERSION = "1.0020";
 
 use Net::Jabber::Protocol;
 ($Net::Jabber::Protocol::VERSION < $VERSION) &&
@@ -147,13 +147,6 @@ sub new {
   
   $self->{LIST}->{currentID} = 0;
 
-  if (eval "require Digest::SHA1") {
-    $self->{DIGEST} = 1;
-    Digest::SHA1->import(qw(sha1 sha1_hex sha1_base64));
-  } else {
-    $self->{DIGEST} = 0;
-  }
-
   return $self;
 }
 
@@ -195,12 +188,13 @@ sub Connect {
 	      port=>$self->{SERVER}->{port},
 	      namespace=>"jabber:client",
 	      ssl=>$self->{SERVER}->{ssl},
+	      timeout=>10
 	     ) || ($self->SetErrorCode($self->{STREAM}->GetErrorCode()) &&
 		   return);
   
   $self->{DEBUG}->Log1("Connect: connection made");
 
-  $self->{STREAM}->OnNode(sub{ $self->CallBack(@_) });
+  $self->{STREAM}->SetCallBacks(node=>sub{ $self->CallBack(@_) });
   $self->{CONNECTED} = 1;
   return 1;
 }
