@@ -1,4 +1,4 @@
-##############################################################################
+#############################################################################
 #
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Library General Public
@@ -35,13 +35,18 @@ Net::Jabber::Presence - Jabber Presence Module
 =head1 DESCRIPTION
 
   To initialize the Presence with a Jabber <presence/> you must pass it 
-  the XML::Parser Tree array from the Net::Jabber::Client module.  In the
-  callback function for the presence:
+  the XML::Parser Tree array.  For example:
+
+    my $presence = new Net::Jabber::Presence(@tree);
+
+  There has been a change from the old way of handling the callbacks.
+  You no longer have to do the above, a Net::Jabber::Presence object is 
+  passed to the callback function for the presence:
 
     use Net::Jabber;
 
     sub presence {
-      my $presence = new Net::Jabber::Presence(@_);
+      my ($Pres) = @_;
       .
       .
       .
@@ -67,12 +72,6 @@ Net::Jabber::Presence - Jabber Presence Module
     $toJID      = $Pres->GetTo("jid");
     $from       = $Pres->GetFrom();
     $fromJID    = $Pres->GetFrom("jid");
-    $sto        = $Pres->GetSTo();
-    $stoJID     = $Pres->GetSTo("jid");
-    $sfrom      = $Pres->GetSFrom();
-    $sfromJID   = $Pres->GetSFrom("jid");
-    $etherxTo   = $Pres->GetEtherxTo();
-    $etherxFrom = $Pres->GetEtherxFrom();
     $type       = $Pres->GetType();
     $status     = $Pres->GetStatus();
     $priority   = $Pres->GetPriority();
@@ -92,10 +91,6 @@ Net::Jabber::Presence - Jabber Presence Module
 		       iCoN=>"normal");
     $Pres->SetTo("bob\@jabber.org");
     $Pres->SetFrom("jojo\@jabber.org");
-    $Pres->SetSTo("jabber.org");
-    $Pres->SetSFrom("jabber.org");
-    $Pres->SetEtherxTo("jabber.org");
-    $Pres->SetEtherxFrom("transport.jabber.org");
     $Pres->SetType("unavailable");
     $Pres->SetStatus("Taking a nap");
     $Pres->SetPriority(10);
@@ -105,18 +100,12 @@ Net::Jabber::Presence - Jabber Presence Module
     $X = $Pres->NewX("my:namespace");
 
     $Reply = $Pres->Reply();
-    $Reply = $Pres->Reply(template=>"client");
-    $Reply = $Pres->Reply(template=>"transport",
-                          type=>"subscribed");
+    $Reply = $Pres->Reply(type=>"subscribed");
 
 =head2 Test functions
 
     $test = $Pres->DefinedTo();
     $test = $Pres->DefinedFrom();
-    $test = $Pres->DefinedSTo();
-    $test = $Pres->DefinedSFrom();
-    $test = $Pres->DefinedEtherxTo();
-    $test = $Pres->DefinedEtherxFrom();
     $test = $Pres->DefinedType();
     $test = $Pres->DefinedStatus();
     $test = $Pres->DefinedPriority();
@@ -138,26 +127,6 @@ Net::Jabber::Presence - Jabber Presence Module
                     the string to "jid", otherwise leave blank for the 
                     text string.
 
-  GetSTo()      - returns either a string with the Jabber Identifier,
-  GetSTo("jid")   or a Net::Jabber::JID object for the <host/> component
-                  who is going to receive the <presence/>.  To get the 
-                  JID object set the string to "jid", otherwise leave
-                  blank for the text string.
-
-  GetSFrom()      -  returns either a string with the Jabber Identifier,
-  GetSFrom("jid")    or a Net::Jabber::JID object for the <host/> 
-                     component who sent the <presence/>.  To get the 
-                     JID object set the string to "jid", otherwise 
-                     leave blank for the text string.
-
-  GetEtherxTo(string) - returns the etherx:to attribute.  This is for
-                        Transport writers who need to communicate with
-                        Etherx.
-
-  GetEtherxFrom(string) -  returns the etherx:from attribute.  This is for
-                           Transport writers who need to communicate with
-                           Etherx.
-
   GetType() - returns a string with the type <presence/> this is.
 
   GetStatus() - returns a string with the current status of the resource.
@@ -191,17 +160,13 @@ Net::Jabber::Presence - Jabber Presence Module
 
   SetPresence(to=>string|JID     - set multiple fields in the <presence/>
               from=>string|JID,    at one time.  This is a cumulative
-              sto=>string,         and over writing action.  If you set
-              sfrom=>string,       the "to" attribute twice, the second
-              etherxto=>string,    setting is what is used.  If you set
-              etherxfrom=>string,  the status, and then set the priority
-              type=>string,        then both will be in the <presence/>
-              status=>string,      tag.  For valid settings read the
-              priority=>integer,   specific Set functions below.
-	      meta=>string, 
-	      icon=>string,
-              show=>string,
-              loc=>string)
+              type=>string,        and over writing action.  If you set
+              status=>string,      the "to" attribute twice, the second
+              priority=>integer,   setting is what is used.  If you set
+              meta=>string,        the status, and then set the priority
+              icon=>string,        then both will be in the <presence/>
+              show=>string,        tag.  For valid settings read the
+              loc=>string)         specific Set functions below.
 
   SetTo(string) - sets the to attribute.  You can either pass a string
   SetTo(JID)      or a JID object.  They must be valid Jabber 
@@ -213,24 +178,6 @@ Net::Jabber::Presence - Jabber Presence Module
                     Identifiers or the server will return an error message.
                     (ie.  jabber:bob@jabber.org/Silent Bob, etc...)
 
-  SetSTo(string) - sets the sto attribute.  You can either pass a string
-  SetSTo(JID)      or a JID object.  They must be valid Jabber 
-                   Identifiers or the server will return an error message.
-                   (ie.  jabber:bob@jabber.org/Silent Bob, etc...)
-
-  SetSFrom(string) - sets the sfrom attribute.  You can either pass a string
-  SetSFrom(JID)      or a JID object.  They must be valid Jabber 
-                     Identifiers or the server will return an error message.
-                     (ie.  jabber:bob@jabber.org/Silent Bob, etc...)
-
-  SetEtherxTo(string) - sets the etherx:to attribute.  This is for
-                        Transport writers who need to communicate with
-                        Etherx.
-
-  SetEtherxFrom(string) -  sets the etherx:from attribute.  This is for
-                           Transport writers who need to communicate with
-                           Etherx.
-
   SetType(string) - sets the type attribute.  Valid settings are:
 
                     available       available to receive messages; default
@@ -257,21 +204,9 @@ Net::Jabber::Presence - Jabber Presence Module
                  namespace with the X module.  For more information
                  please read the documentation for Net::Jabber::X.
 
-  Reply(template=>string, - creates a new Presence object and
-        type=>string)       populates the to/from and
-                            etherxto/etherxfrom fields based
-                            the value of template.  The following
-                            templates are available:
-
-                            client: (default)
-                                 just sets the to/from
-
-                            transport:
-                            transport-reply:
-                                 the transport will send the
-                                 reply to the sender
-
-                            The type will be set in the <presence/>.
+  Reply(type=>string) - creates a new Presence object and populates
+                        the to/from fields.  The type will be set in 
+                        the <presence/>.
 
 =head2 Test functions
 
@@ -281,12 +216,6 @@ Net::Jabber::Presence - Jabber Presence Module
   DefinedFrom() - returns 1 if the from attribute is defined in the 
                   <presence/>, 0 otherwise.
 
-  DefinedEtherxTo() - returns 1 if the etherx:to attribute is defined in 
-                      the <presence/>, 0 otherwise.
-
-  DefinedEtherxFrom() - returns 1 if the etherx:from attribute is defined 
-                        in the <presence/>, 0 otherwise.
-
   DefinedType() - returns 1 if the type attribute is defined in the 
                   <presence/>, 0 otherwise.
 
@@ -315,7 +244,7 @@ use strict;
 use Carp;
 use vars qw($VERSION $AUTOLOAD %FUNCTIONS);
 
-$VERSION = "1.0019";
+$VERSION = "1.0020";
 
 sub new {
   my $proto = shift;
@@ -330,13 +259,17 @@ sub new {
 					  header=>"NJ::Presence");
 
   if ("@_" ne ("")) {
-    my @temp = @_;
-    $self->{PRESENCE} = \@temp;
-    my $xTree;
-    foreach $xTree ($self->GetXTrees()) {
-      my $xmlns = &Net::Jabber::GetXMLData("value",$xTree,"","xmlns");
-      next if !exists($Net::Jabber::DELEGATES{x}->{$xmlns});
-      $self->AddX($xmlns,@{$xTree}) if ($xmlns ne "");
+    if (ref($_[0]) eq "Net::Jabber::Presence") {
+      return $_[0];
+    } else {
+      my @temp = @_;
+      $self->{PRESENCE} = \@temp;
+      my $xTree;
+      foreach $xTree ($self->GetXTrees()) {
+	my $xmlns = &Net::Jabber::GetXMLData("value",$xTree,"","xmlns");
+	next if !exists($Net::Jabber::DELEGATES{x}->{$xmlns});
+	$self->AddX($xmlns,@{$xTree}) if ($xmlns ne "");
+      }
     }
   } else {
     $self->{PRESENCE} = [ "presence" , [{}] ];
@@ -374,17 +307,11 @@ sub AUTOLOAD {
 
 $FUNCTIONS{get}->{To}         = ["value","","to"];
 $FUNCTIONS{get}->{From}       = ["value","","from"];
-$FUNCTIONS{get}->{STo}        = ["value","","sto"];
-$FUNCTIONS{get}->{SFrom}      = ["value","","sfrom"];
-$FUNCTIONS{get}->{EtherxTo}   = ["value","","etherx:to"];
-$FUNCTIONS{get}->{EtherxFrom} = ["value","","etherx:from"];
 $FUNCTIONS{get}->{ID}         = ["value","","id"];
 $FUNCTIONS{get}->{Type}       = ["value","","type"];
 $FUNCTIONS{get}->{Status}     = ["value","status",""];
 $FUNCTIONS{get}->{Show}       = ["value","show",""];
 
-$FUNCTIONS{set}->{EtherxTo}   = ["single","","","etherx:to","*"];
-$FUNCTIONS{set}->{EtherxFrom} = ["single","","","etherx:from","*"];
 $FUNCTIONS{set}->{ID}         = ["single","","","id","*"];
 $FUNCTIONS{set}->{Type}       = ["single","","","type","*"];
 $FUNCTIONS{set}->{Status}     = ["single","status","*","",""];
@@ -392,10 +319,6 @@ $FUNCTIONS{set}->{Show}       = ["single","show","*","",""];
 
 $FUNCTIONS{defined}->{To}         = ["existence","","to"];
 $FUNCTIONS{defined}->{From}       = ["existence","","from"];
-$FUNCTIONS{defined}->{STo}        = ["existence","","sto"];
-$FUNCTIONS{defined}->{SFrom}      = ["existence","","sfrom"];
-$FUNCTIONS{defined}->{EtherxTo}   = ["existence","","etherx:to"];
-$FUNCTIONS{defined}->{EtherxFrom} = ["existence","","etherx:from"];
 $FUNCTIONS{defined}->{ID}         = ["existence","","id"];
 $FUNCTIONS{defined}->{Type}       = ["existence","","type"];
 $FUNCTIONS{defined}->{Status}     = ["existence","status",""];
@@ -492,10 +415,6 @@ sub SetPresence {
   $self->SetID($presence{id}) if exists($presence{id});
   $self->SetTo($presence{to}) if exists($presence{to});
   $self->SetFrom($presence{from}) if exists($presence{from});
-  $self->SetSTo($presence{sto}) if exists($presence{sto});
-  $self->SetSFrom($presence{sfrom}) if exists($presence{sfrom});
-  $self->SetEtherxTo($presence{etherxto}) if exists($presence{etherxto});
-  $self->SetEtherxFrom($presence{etherxfrom}) if exists($presence{etherxfrom});
   $self->SetType($presence{type}) if exists($presence{type});
   $self->SetStatus($presence{status}) if exists($presence{status});
   $self->SetPriority($presence{priority}) if exists($presence{priority});
@@ -514,6 +433,7 @@ sub SetTo {
   if (ref($to) eq "Net::Jabber::JID") {
     $to = $to->GetJID("full");
   }
+  return unless ($to ne "");
   &Net::Jabber::SetXMLData("single",$self->{PRESENCE},"","",{to=>$to});
 }
 
@@ -529,40 +449,11 @@ sub SetFrom {
   if (ref($from) eq "Net::Jabber::JID") {
     $from = $from->GetJID("full");
   }
+  return unless ($from ne "");
   &Net::Jabber::SetXMLData("single",$self->{PRESENCE},"","",{from=>$from});
 }
 
 
-##############################################################################
-#
-# SetSTo - sets the sto attribute in the <presence/>
-#
-##############################################################################
-sub SetSTo {
-  my $self = shift;
-  my ($sto) = @_;
-  if (ref($sto) eq "Net::Jabber::JID") {
-    $sto = $sto->GetJID("full");
-  }
-  &Net::Jabber::SetXMLData("single",$self->{PRESENCE},"","",{sto=>$sto});
-}
-
-
-##############################################################################
-#
-# SetSFrom - sets the sfrom attribute in the <presence/>
-#
-##############################################################################
-sub SetSFrom {
-  my $self = shift;
-  my ($sfrom) = @_;
-  if (ref($sfrom) eq "Net::Jabber::JID") {
-    $sfrom = $sfrom->GetJID("full");
-  }
-  &Net::Jabber::SetXMLData("single",$self->{PRESENCE},"","",{sfrom=>$sfrom});
-}
-
-
 ##############################################################################
 #
 # SetPriority - sets the priority of the <presence/>
@@ -685,27 +576,13 @@ sub Reply {
   while($#_ >= 0) { $args{ lc pop(@_) } = pop(@_); }
 
   my $reply = new Net::Jabber::Presence();
-
+  
   $reply->SetID($self->GetID()) if ($self->GetID() ne "");
   $reply->SetType(exists($args{type}) ? $args{type} : "");
-
-  if (exists($args{template})) {
-    if ($args{template} eq "transport") {
-      my $fromJID = $self->GetFrom("jid");
-      
-      $reply->SetPresence(to=>$self->GetFrom(),
-			  from=>$self->GetTo(),
-			  etherxto=>$fromJID->GetServer(),
-			  etherxfrom=>$self->GetEtherxTo(),
-			 );
-    } else {
-      $reply->SetPresence(to=>$self->GetFrom(),
-			  from=>$self->GetTo());
-    }
-  } else {
-    $reply->SetPresence(to=>$self->GetFrom(),
-			from=>$self->GetTo());
-  }
+  
+  $reply->SetPresence(to=>$self->GetFrom(),
+		      from=>$self->GetTo(),
+		     );
 
   return $reply;
 }
